/*----------------------------------------------------------------------------
* Testprogramm fuer ubasic_avr-Interpreter....
* 
* Uwe Berger; 2010
* 
* 
* Ein-/Ausgaben erfolgen ueber die serielle Schnittstelle (9600, 8, n, 1)
*
* Kommandos:
*        list --> gibt das aktuelle Basic-Programm aus
*        run  --> startet das aktuell geladene Basic-Programm
*        load --> die folgenden Eingaben werden als Basic-Programm gewertet
*                 mit einem '#' beendet man den Eingabe-Mode
*
* Have fun!
* ---------
* 
* ----------------------------------------------------------------------------*/

#include <avr/io.h>
#include "usart.h"
#include "ubasic.h"

//Taktfrequenz
#ifndef F_CPU
#define F_CPU 16000000UL
#endif

//Baudrate der seriellen Schnittstelle
#define BAUDRATE 57600


#if 0
static const char program[] =
"10 gosub 100\n\
15 a=30\n\
20 for i = 1 to a step 10\n\
30 print \"i=\", i\n\
40 next i\n\
50 end\n\
100 print \"subroutine\"\n\
110 return\n";
#endif

#if 0
static const char program[] =
"5  c=9\n\
 10 print \"davor\", c\n\
 20 print in(c,0)\n\
 30 print \"danach\"\n\
 60 end\n";
#endif

#if 0
static char program[] =
"10 epoke(2)=55\n\
 25 epoke(3)=11\n\
 20 a=epeek(2)\n\
 25 print a\n\
 26 print epeek(3)+3\n\
 30 print \"end\"\n\
 40 end\n";
#endif

#if 0
static const char program[] =
"10 a=1\n\
20 print a\n\
30 wait 1000\n\
40 a=a+1\n\
50 goto 20\n\
60 end\n";
#endif

#if 0
static const char program[] =
"10 dir(b,1)=1\n\
 20 a=0\n\
 30 out(b,1)=a\n\
 40 if a=1 then a=0 else a=1\n\
 50 wait 1000\n\
 60 goto 30\n\
 70 end\n";
#endif

#if 0
static const char program[] =
"10 dir(b,1)=1\n\
 20 out(b,1)=1\n\
 30 wait 1000\n\
 40 out(b,1)=0\n\
 50 wait 1000\n\
 60 goto 20\n\
 70 end\n";
#endif

#if 0
static const char program[] =
"10 dir(b,1)=1\n\
 20 a=0\n\
 30 a=a+1\n\
 40 if (a%2)=1 then out(b,1)=0 else out(b,1)=1\n\
 50 print \"a=\",a,\"adc(0)=\",adc(0)\n\
 50 wait 1000\n\
 60 goto 30\n\
 70 end\n";
#endif

#if 0
static const char program[] =
"10 print abs(-8)\n\
 20 print not(7)\n\
 30 end\n";
#endif

#if 1
static const char program[] =
"\
10 srand\n\
20 for v = 1 to 2000\n\
30 y=rand(9)\n\
40 if y=0 then a=a+1\n\
50 if y=1 then b=b+1\n\
60 if y=2 then c=c+1\n\
70 if y=3 then d=d+1\n\
80 if y=4 then e=e+1\n\
90 if y=5 then f=f+1\n\
90 if y=6 then g=g+1\n\
90 if y=7 then h=h+1\n\
90 if y=8 then i=i+1\n\
90 if y=9 then j=j+1\n\
100 next v\n\
110 print a,b,c,d,e,f,g,h,i,j\n\
120 end\n\
";
#endif

#if 0
static const char program[] =
"\
40 if 0=0 then print 0\n\
100 print 1\n\
120 end\n\
";
#endif


//----------------------------------------------------------------------------
void basic_load( char* Buffer, int BufferLen ){
	int i = 0;
	char c;

	while((c=usart_receive_char()) != '#' && i < BufferLen - 1) {
		if (c=='\r' || c=='\n') {
			c='\n';
			usart_write("\r\n");
		} else {
			usart_write_char(c);
		}
		Buffer[i++]=c;
	}
	Buffer[i] = '\0';
	usart_write_str("\r\nEnde Eingabemodus...\r\n");
}

//----------------------------------------------------------------------------
void basic_list(char *c) {
	while (*c) {
		if (*c == '\n') usart_write_char('\r');
		usart_write_char(*c++);
	}
}


//----------------------------------------------------------------------------
//Hier startet das Hauptprogramm
int main(void)
{  
	char buffer[24];
	
    usart_init(BAUDRATE); // setup the UART
	
	usart_write("AVR-uBasic-Interpreter; Uwe Berger, 2010\r\n");
	usart_write("Compiliert am "__DATE__" um "__TIME__"\r\n");
	usart_write("Compiliert mit GCC Version "__VERSION__"\r\n");
	
	
	while(1){

		char* command = buffer;
        if(usart_read_line(command, sizeof(buffer)) < 1) continue;
        // Kommandos auswerten
        if (strcmp_P(command, PSTR("run")) == 0) {
			usart_write_str("\r\n");
			ubasic_init(program);
			do {
				ubasic_run();
			} while(!ubasic_finished());
        } else
        if (strcmp_P(command, PSTR("load")) == 0) {
			usart_write_str("\r\nBeginn Eingabemodus...\r\n");
			basic_load(program, 300);			
        } else
        if (strcmp_P(command, PSTR("list")) == 0) {
			usart_write_str("\r\n");
			basic_list(program);
        } else 	usart_write("\r\nKommando %s unbekannt!\r\n", command);
    }
		
return(0);
}

